@@ -43,6 +43,8 @@ unsigned exam_indexed();
 unsigned exam_color();
 unsigned exam_clifford();
 unsigned exam_archive();
+unsigned exam_hashmap();
 unsigned exam_misc();
+unsigned exam_inifcns_nstdsums();
 
 #endif // ndef EXAMS_H
