@@ -65,10 +65,6 @@ public:
 	struct property_info {
 		property_info() {}
 		property_info(property_type t, const std::string &n, unsigned c = 1) : type(t), name(n), count(c) {}
-		~property_info() {}
-
-		property_info(const property_info &other) : type(other.type), name(other.name), count(other.count) {}
-		const property_info &operator=(const property_info &other);
 
 		property_type type;	/**< Data type of property. */
 		std::string name;   /**< Name of property. */
@@ -79,9 +75,7 @@ public:
 	archive_node() : a(*dummy_ar_creator()), has_expression(false) {} // hack for cint which always requires a default constructor
 	archive_node(archive &ar) : a(ar), has_expression(false) {}
 	archive_node(archive &ar, const ex &expr);
-	~archive_node() {}
 
-	archive_node(const archive_node &other);
 	const archive_node &operator=(const archive_node &other);
 
 	/** Add property of type "bool" to node. */
@@ -110,7 +104,7 @@ public:
 
 	/** Retrieve property of type "ex" from node.
 	 *  @return "true" if property was found, "false" otherwise */
-	bool find_ex(const std::string &name, ex &ret, const lst &sym_lst, unsigned index = 0) const;
+	bool find_ex(const std::string &name, ex &ret, lst &sym_lst, unsigned index = 0) const;
 
 	/** Retrieve property of type "ex" from node, returning the node of
 	 *  the sub-expression. */
@@ -119,7 +113,7 @@ public:
 	/** Return vector of properties stored in node. */
 	void get_properties(propinfovector &v) const;
 
-	ex unarchive(const lst &sym_lst) const;
+	ex unarchive(lst &sym_lst) const;
 	bool has_same_ex_as(const archive_node &other) const;
 
 	void forget(void);
@@ -132,10 +126,6 @@ private:
 	struct property {
 		property() {}
 		property(archive_atom n, property_type t, unsigned v) : type(t), name(n), value(v) {}
-		~property() {}
-
-		property(const property &other) : type(other.type), name(other.name), value(other.value) {}
-		const property &operator=(const property &other);
 
 		property_type type; /**< Data type of property. */
 		archive_atom name;  /**< Name of property. */
@@ -184,17 +174,20 @@ public:
 	void archive_ex(const ex &e, const char *name);
 
 	/** Retrieve expression from archive by name.
-	 *  @param sym_lst list of pre-defined symbols */
+	 *  @param sym_lst list of pre-defined symbols
+	 *  @param name name of expression */
 	ex unarchive_ex(const lst &sym_lst, const char *name) const;
 
 	/** Retrieve expression from archive by index.
 	 *  @param sym_lst list of pre-defined symbols
+	 *  @param index index of expression
      *  @see count_expressions */
 	ex unarchive_ex(const lst &sym_lst, unsigned index = 0) const;
 
 	/** Retrieve expression and its name from archive by index.
 	 *  @param sym_lst list of pre-defined symbols
 	 *  @param name receives the name of the expression
+	 *  @param index index of expression
      *  @see count_expressions */
 	ex unarchive_ex(const lst &sym_lst, std::string &name, unsigned index = 0) const;
 
