@@ -3,7 +3,7 @@
  *  Assertion macro definition. */
 
 /*
- *  GiNaC Copyright (C) 1999-2000 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2003 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,12 +23,13 @@
 #ifndef __GINAC_ASSERTION_H__
 #define __GINAC_ASSERTION_H__
 
-#include <assert.h>
-
 #if !defined(GINAC_ASSERT)
 #if defined(DO_GINAC_ASSERT)
+#include <cassert>
+/** Assertion macro for checking invariances. */
 #define GINAC_ASSERT(X) assert(X)
 #else
+/** Assertion macro for checking invariances. */
 #define GINAC_ASSERT(X) ((void)0)
 #endif
 #endif
