@@ -23,6 +23,7 @@
 #ifndef __GINAC_BASIC_H__
 #define __GINAC_BASIC_H__
 
+#include <cstddef> // for size_t
 #include <vector>
 // CINT needs <algorithm> to work properly with <vector>
 #include <algorithm>
@@ -35,6 +36,7 @@
 namespace GiNaC {
 
 class ex;
+class ex_is_less;
 class symbol;
 class lst;
 class numeric;
@@ -95,11 +97,13 @@ public: // only const functions please (may break reference counting)
 	virtual void dbgprinttree(void) const;
 	virtual unsigned precedence(void) const;
 	virtual bool info(unsigned inf) const;
-	virtual unsigned nops() const;
-	virtual ex op(int i) const;
-	virtual ex & let_op(int i);
+	virtual size_t nops() const;
+	virtual ex op(size_t i) const;
 	virtual ex operator[](const ex & index) const;
-	virtual ex operator[](int i) const;
+	virtual ex operator[](size_t i) const;
+	virtual ex & let_op(size_t i);
+	virtual ex & operator[](const ex & index);
+	virtual ex & operator[](size_t i);
 	virtual ex expand(unsigned options = 0) const;
 	virtual bool has(const ex & other) const;
 	virtual ex map(map_function & f) const;
@@ -112,9 +116,10 @@ public: // only const functions please (may break reference counting)
 	virtual ex evalm(void) const;
 	virtual ex series(const relational & r, int order, unsigned options = 0) const;
 	virtual bool match(const ex & pattern, lst & repl_lst) const;
-	virtual ex subs(const lst & ls, const lst & lr, bool no_pattern = false) const;
+	virtual ex subs(const lst & ls, const lst & lr, unsigned options = 0) const;
 	virtual ex normal(lst &sym_lst, lst &repl_lst, int level = 0) const;
 	virtual ex to_rational(lst &repl_lst) const;
+	virtual ex to_polynomial(lst &repl_lst) const;
 	virtual numeric integer_content(void) const;
 	virtual ex smod(const numeric &xi) const;
 	virtual numeric max_coefficient(void) const;
@@ -131,12 +136,12 @@ protected: // functions that should be called from class ex only
 	virtual bool is_equal_same_type(const basic & other) const;
 	virtual bool match_same_type(const basic & other) const;
 	virtual unsigned calchash(void) const;
-	virtual ex simplify_ncmul(const exvector & v) const;
+	virtual ex eval_ncmul(const exvector & v) const;
 	
 	// non-virtual functions in this class
 public:
-	ex subs(const ex & e, bool no_pattern = false) const;
-	ex diff(const symbol & s, unsigned nth=1) const;
+	ex subs(const ex & e, unsigned options = 0) const;
+	ex diff(const symbol & s, unsigned nth = 1) const;
 	int compare(const basic & other) const;
 	bool is_equal(const basic & other) const;
 	const basic & hold(void) const;
@@ -171,7 +176,7 @@ extern int max_recursion_level;
 template <class T>
 inline bool is_a(const basic &obj)
 {
-	return dynamic_cast<const T *>(&obj)!=0;
+	return dynamic_cast<const T *>(&obj) != 0;
 }
 
 /** Check if obj is a T, not including base classes.  This one is just an
@@ -180,36 +185,7 @@ inline bool is_a(const basic &obj)
 template <class T>
 inline bool is_exactly_a(const class basic &obj)
 {
-	const T foo; return foo.tinfo()==obj.tinfo();
-}
-
-/** Check if ex is a handle to a T, including base classes. */
-template <class T>
-inline bool is_a(const ex &obj)
-{
-	return is_a<T>(*obj.bp);
-}
-
-/** Check if ex is a handle to a T, not including base classes. */
-template <class T>
-inline bool is_exactly_a(const ex &obj)
-{
-	return is_exactly_a<T>(*obj.bp);
-}
-
-/** Return a reference to the basic-derived class T object embedded in an
- *  expression.  This is fast but unsafe: the result is undefined if the
- *  expression does not contain a T object at its top level.  Hence, you
- *  should generally check the type of e first.
- *
- *  @param e expression
- *  @return reference to pseries object
- *  @see is_exactly_a<class T>() */
-template <class T>
-inline const T &ex_to(const ex &e)
-{
-	GINAC_ASSERT(is_a<T>(e));
-	return static_cast<const T &>(*e.bp);
+	return obj.tinfo() == T::reg_info.tinfo_key;
 }
 
 } // namespace GiNaC
