@@ -32,6 +32,7 @@
 #include "symmetry.h"
 #include "lst.h"
 #include "relational.h"
+#include "operators.h"
 #include "mul.h"
 #include "print.h"
 #include "archive.h"
@@ -103,7 +104,7 @@ clifford::clifford(unsigned char rl, exvector * vp) : inherited(sy_none(), vp),
 // archiving
 //////////
 
-clifford::clifford(const archive_node &n, const lst &sym_lst) : inherited(n, sym_lst)
+clifford::clifford(const archive_node &n, lst &sym_lst) : inherited(n, sym_lst)
 {
 	unsigned rl;
 	n.find_unsigned("label", rl);
@@ -295,7 +296,7 @@ bool diracgamma::contract_with(exvector::iterator self, exvector::iterator other
 /** Perform automatic simplification on noncommutative product of clifford
  *  objects. This removes superfluous ONEs, permutes gamma5/L/R's to the front
  *  and removes squares of gamma objects. */
-ex clifford::simplify_ncmul(const exvector & v) const
+ex clifford::eval_ncmul(const exvector & v) const
 {
 	exvector s;
 	s.reserve(v.size());
@@ -453,9 +454,9 @@ ex clifford::simplify_ncmul(const exvector & v) const
 	if (s.empty())
 		return clifford(diracone(), representation_label) * sign;
 	if (something_changed)
-		return nonsimplified_ncmul(s) * sign;
+		return reeval_ncmul(s) * sign;
 	else
-		return simplified_ncmul(s) * sign;
+		return hold_ncmul(s) * sign;
 }
 
 ex clifford::thisexprseq(const exvector & v) const
@@ -500,16 +501,6 @@ ex dirac_gammaR(unsigned char rl)
 	return clifford(diracgammaR(), rl);
 }
 
-ex dirac_gamma6(unsigned char rl)
-{
-	return clifford(diracone(), rl) + clifford(diracgamma5(), rl);
-}
-
-ex dirac_gamma7(unsigned char rl)
-{
-	return clifford(diracone(), rl) - clifford(diracgamma5(), rl);
-}
-
 ex dirac_slash(const ex & e, const ex & dim, unsigned char rl)
 {
 	// Slashed vectors are actually stored as a clifford object with the
@@ -534,7 +525,7 @@ static bool is_clifford_tinfo(unsigned ti)
 
 /** Take trace of a string of an even number of Dirac gammas given a vector
  *  of indices. */
-static ex trace_string(exvector::const_iterator ix, unsigned num)
+static ex trace_string(exvector::const_iterator ix, size_t num)
 {
 	// Tr gamma.mu gamma.nu = 4 g.mu.nu
 	if (num == 2)
@@ -556,8 +547,8 @@ static ex trace_string(exvector::const_iterator ix, unsigned num)
 	exvector v(num - 2);
 	int sign = 1;
 	ex result;
-	for (unsigned i=1; i<num; i++) {
-		for (unsigned n=1, j=0; n<num; n++) {
+	for (size_t i=1; i<num; i++) {
+		for (size_t n=1, j=0; n<num; n++) {
 			if (n == i)
 				continue;
 			v[j++] = ix[n];
@@ -582,11 +573,11 @@ ex dirac_trace(const ex & e, unsigned char rl, const ex & trONE)
 		else
 			return _ex0;
 
-	} else if (is_ex_exactly_of_type(e, mul)) {
+	} else if (is_exactly_a<mul>(e)) {
 
 		// Trace of product: pull out non-clifford factors
 		ex prod = _ex1;
-		for (unsigned i=0; i<e.nops(); i++) {
+		for (size_t i=0; i<e.nops(); i++) {
 			const ex &o = e.op(i);
 			if (is_clifford_tinfo(o.return_type_tinfo(), rl))
 				prod *= dirac_trace(o, rl, trONE);
@@ -595,7 +586,7 @@ ex dirac_trace(const ex & e, unsigned char rl, const ex & trONE)
 		}
 		return prod;
 
-	} else if (is_ex_exactly_of_type(e, ncmul)) {
+	} else if (is_exactly_a<ncmul>(e)) {
 
 		if (!is_clifford_tinfo(e.return_type_tinfo(), rl))
 			return _ex0;
@@ -604,13 +595,13 @@ ex dirac_trace(const ex & e, unsigned char rl, const ex & trONE)
 		ex e_expanded = e.subs(lst(
 			dirac_gammaL(rl) == (dirac_ONE(rl)-dirac_gamma5(rl))/2,
 			dirac_gammaR(rl) == (dirac_ONE(rl)+dirac_gamma5(rl))/2
-		)).expand();
+		), subs_options::no_pattern).expand();
 		if (!is_a<ncmul>(e_expanded))
 			return dirac_trace(e_expanded, rl, trONE);
 
 		// gamma5 gets moved to the front so this check is enough
 		bool has_gamma5 = is_a<diracgamma5>(e.op(0).op(0));
-		unsigned num = e.nops();
+		size_t num = e.nops();
 
 		if (has_gamma5) {
 
@@ -634,23 +625,23 @@ ex dirac_trace(const ex & e, unsigned char rl, const ex & trONE)
 			//   I/4! * epsilon0123.mu1.mu2.mu3.mu4 * Tr gamma.mu1 gamma.mu2 gamma.mu3 gamma.mu4 S_2k
 			// (the epsilon is always 4-dimensional)
 			exvector ix(num-1), bv(num-1);
-			for (unsigned i=1; i<num; i++)
+			for (size_t i=1; i<num; i++)
 				base_and_index(e.op(i), bv[i-1], ix[i-1]);
 			num--;
 			int *iv = new int[num];
 			ex result;
-			for (unsigned i=0; i<num-3; i++) {
+			for (size_t i=0; i<num-3; i++) {
 				ex idx1 = ix[i];
-				for (unsigned j=i+1; j<num-2; j++) {
+				for (size_t j=i+1; j<num-2; j++) {
 					ex idx2 = ix[j];
-					for (unsigned k=j+1; k<num-1; k++) {
+					for (size_t k=j+1; k<num-1; k++) {
 						ex idx3 = ix[k];
-						for (unsigned l=k+1; l<num; l++) {
+						for (size_t l=k+1; l<num; l++) {
 							ex idx4 = ix[l];
 							iv[0] = i; iv[1] = j; iv[2] = k; iv[3] = l;
 							exvector v;
 							v.reserve(num - 4);
-							for (unsigned n=0, t=4; n<num; n++) {
+							for (size_t n=0, t=4; n<num; n++) {
 								if (n == i || n == j || n == k || n == l)
 									continue;
 								iv[t++] = n;
@@ -681,7 +672,7 @@ ex dirac_trace(const ex & e, unsigned char rl, const ex & trONE)
 			}
 
 			exvector iv(num), bv(num);
-			for (unsigned i=0; i<num; i++)
+			for (size_t i=0; i<num; i++)
 				base_and_index(e.op(i), bv[i], iv[i]);
 
 			return trONE * (trace_string(iv.begin(), num) * mul(bv)).simplify_indexed();
@@ -702,19 +693,20 @@ ex canonicalize_clifford(const ex & e)
 	// Scan for any ncmul objects
 	lst srl;
 	ex aux = e.to_rational(srl);
-	for (unsigned i=0; i<srl.nops(); i++) {
+	for (size_t i=0; i<srl.nops(); i++) {
 
-		ex lhs = srl.op(i).lhs();
-		ex rhs = srl.op(i).rhs();
+		ex o = srl.op(i);
+		ex lhs = o.lhs();
+		ex rhs = o.rhs();
 
-		if (is_ex_exactly_of_type(rhs, ncmul)
+		if (is_exactly_a<ncmul>(rhs)
 		 && rhs.return_type() == return_types::noncommutative
 		 && is_clifford_tinfo(rhs.return_type_tinfo())) {
 
 			// Expand product, if necessary
 			ex rhs_expanded = rhs.expand();
 			if (!is_a<ncmul>(rhs_expanded)) {
-				srl.let_op(i) = (lhs == canonicalize_clifford(rhs_expanded));
+				srl[i] = (lhs == canonicalize_clifford(rhs_expanded));
 				continue;
 
 			} else if (!is_a<clifford>(rhs.op(0)))
@@ -722,7 +714,7 @@ ex canonicalize_clifford(const ex & e)
 
 			exvector v;
 			v.reserve(rhs.nops());
-			for (unsigned j=0; j<rhs.nops(); j++)
+			for (size_t j=0; j<rhs.nops(); j++)
 				v.push_back(rhs.op(j));
 
 			// Stupid recursive bubble sort because we only want to swap adjacent gammas
@@ -741,7 +733,7 @@ ex canonicalize_clifford(const ex & e)
 					it[0] = save1;
 					it[1] = save0;
 					sum -= ncmul(v, true);
-					srl.let_op(i) = (lhs == canonicalize_clifford(sum));
+					srl[i] = (lhs == canonicalize_clifford(sum));
 					goto next_sym;
 				}
 				++it;
@@ -749,7 +741,7 @@ ex canonicalize_clifford(const ex & e)
 next_sym:	;
 		}
 	}
-	return aux.subs(srl).simplify_indexed();
+	return aux.subs(srl, subs_options::no_pattern).simplify_indexed();
 }
 
 } // namespace GiNaC
