@@ -106,20 +106,36 @@ END_OF_SORT_IMPLEMENTATION
 }
 
 if ($let_op) {
-	$LET_OP_IMPLEMENTATION=<<END_OF_LET_OP_IMPLEMENTATION
-ex & ${CONTAINER}::let_op(int i)
+	$LET_OP_DEFINITION=<<END_OF_LET_OP_DEFINITION;
+	ex & let_op(size_t i);
+END_OF_LET_OP_DEFINITION
+
+	$LET_OP_IMPLEMENTATION=<<END_OF_LET_OP_IMPLEMENTATION;
+ex ${CONTAINER}::op(size_t i) const
 {
-	GINAC_ASSERT(i>=0);
 	GINAC_ASSERT(i<nops());
 
+	${STLT}::const_iterator it=seq.begin();
+	for (size_t j=0; j<i; j++) {
+		++it;
+	}
+	return *it;
+}
+
+ex & ${CONTAINER}::let_op(size_t i)
+{
+	GINAC_ASSERT(i<nops());
+
+	ensure_if_modifiable();
 	${STLT}::iterator it=seq.begin();
-	for (int j=0; j<i; j++) {
+	for (size_t j=0; j<i; j++) {
 		++it;
 	}
 	return *it;
 }
 END_OF_LET_OP_IMPLEMENTATION
 } else {
+	$LET_OP_DEFINITION="";
 	$LET_OP_IMPLEMENTATION="// ${CONTAINER}::let_op() will be implemented by user elsewhere";
 }
 
@@ -218,6 +234,7 @@ $interface=<<END_OF_INTERFACE;
 
 #include "basic.h"
 #include "ex.h"
+#include "container_suppl.h"
 
 namespace GiNaC {
 
@@ -228,20 +245,35 @@ class ${CONTAINER} : public basic
 {
 	GINAC_DECLARE_REGISTERED_CLASS(${CONTAINER}, basic)
 
+public:
+	typedef ${STLT}::const_iterator const_iterator;
+	typedef ${STLT}::const_reverse_iterator const_reverse_iterator;
+
 public:
 	${CONTAINER}(${STLT} const & s, bool discardable = false);
 	${CONTAINER}(${STLT} * vp); // vp will be deleted
+	${CONTAINER}(exvector::const_iterator b, exvector::const_iterator e);
 ${constructors_interface}
 
+	// First step of initialization of container with a comma-separated
+	// sequence of expressions. Subsequent steps are handled by
+	// container_init<>::operator,().
+	container_init<ex, ${STLT}> operator=(const ex & x)
+	{
+		this->seq.push_back(x);
+		return container_init<ex, ${STLT}>(seq);
+	}
+
 public:
 	void print(const print_context & c, unsigned level = 0) const;
 	unsigned precedence(void) const {return 10;}
 	bool info(unsigned inf) const;
-	unsigned nops() const;
-	ex & let_op(int i);
+	size_t nops() const;
+	ex op(size_t i) const;
+${LET_OP_DEFINITION}
 	ex map(map_function & f) const;
 	ex eval(int level=0) const;
-	ex subs(const lst & ls, const lst & lr, bool no_pattern = false) const;
+	ex subs(const lst & ls, const lst & lr, unsigned options = 0) const;
 protected:
 	bool is_equal_same_type(const basic & other) const;
 
@@ -249,6 +281,7 @@ protected:
 public:
 	virtual ${CONTAINER} & append(const ex & b);
 	virtual ${CONTAINER} & remove_last(void);
+	virtual ${CONTAINER} & remove_all(void);
 ${PREPEND_INTERFACE}
 ${SORT_INTERFACE}
 protected:
@@ -258,10 +291,17 @@ protected:
 	virtual ex this${CONTAINER}(${STLT} const & v) const;
 	virtual ex this${CONTAINER}(${STLT} * vp) const;
 
+	// non-virtual functions in this class
+public:
+	const_iterator begin() const {return seq.begin();}
+	const_iterator end() const {return seq.end();}
+	const_reverse_iterator rbegin() const {return seq.rbegin();}
+	const_reverse_iterator rend() const {return seq.rend();}
+
 protected:
 	bool is_canonical() const;
 	${STLT} evalchildren(int level) const;
-	${STLT} * subschildren(const lst & ls, const lst & lr, bool no_pattern = false) const;
+	${STLT} * subschildren(const lst & ls, const lst & lr, unsigned options = 0) const;
 
 protected:
 	${STLT} seq;
@@ -368,9 +408,17 @@ ${CONTAINER}::${CONTAINER}(${STLT} const & s, bool discardable) :  basic(TINFO_$
 
 ${CONTAINER}::${CONTAINER}(${STLT} * vp) : basic(TINFO_${CONTAINER})
 {
-	GINAC_ASSERT(vp!=0);
-	seq.swap(*vp);
-	delete vp;
+	if (vp == 0) {
+		// lst(0) ends up here
+		seq.push_back(0);
+	} else {
+		seq.swap(*vp);
+		delete vp;
+	}
+}
+
+${CONTAINER}::${CONTAINER}(exvector::const_iterator b, exvector::const_iterator e) :  basic(TINFO_${CONTAINER}), seq(b, e)
+{
 }
 
 ${constructors_implementation}
@@ -380,7 +428,7 @@ ${constructors_implementation}
 //////////
 
 /** Construct object from archive_node. */
-${CONTAINER}::${CONTAINER}(const archive_node &n, const lst &sym_lst) : inherited(n, sym_lst)
+${CONTAINER}::${CONTAINER}(const archive_node &n, lst &sym_lst) : inherited(n, sym_lst)
 {
 	for (unsigned int i=0; true; i++) {
 		ex e;
@@ -392,7 +440,7 @@ ${CONTAINER}::${CONTAINER}(const archive_node &n, const lst &sym_lst) : inherite
 }
 
 /** Unarchive the object. */
-ex ${CONTAINER}::unarchive(const archive_node &n, const lst &sym_lst)
+ex ${CONTAINER}::unarchive(const archive_node &n, lst &sym_lst)
 {
 	return (new ${CONTAINER}(n, sym_lst))->setflag(status_flags::dynallocated);
 }
@@ -442,7 +490,7 @@ void ${CONTAINER}::print(const print_context & c, unsigned level) const
 
 // ${CONTAINER}::info() will be implemented by user elsewhere";
 
-unsigned ${CONTAINER}::nops() const
+size_t ${CONTAINER}::nops() const
 {
 	return seq.size();
 }
@@ -472,13 +520,13 @@ ex ${CONTAINER}::eval(int level) const
 	return this${CONTAINER}(evalchildren(level));
 }
 
-ex ${CONTAINER}::subs(const lst & ls, const lst & lr, bool no_pattern) const
-{
-	${STLT} *vp = subschildren(ls, lr, no_pattern);
+ex ${CONTAINER}::subs(const lst & ls, const lst & lr, unsigned options) const
+{	
+	${STLT} *vp = subschildren(ls, lr, options);
 	if (vp)
-		return ex_to<basic>(this${CONTAINER}(vp)).basic::subs(ls, lr, no_pattern);
+		return ex_to<basic>(this${CONTAINER}(vp)).basic::subs(ls, lr, options);
 	else
-		return basic::subs(ls, lr, no_pattern);
+		return basic::subs(ls, lr, options);
 }
 
 // protected
@@ -541,6 +589,13 @@ ${CONTAINER} & ${CONTAINER}::remove_last(void)
 	return *this;
 }
 
+${CONTAINER} & ${CONTAINER}::remove_all(void)
+{
+	ensure_if_modifiable();
+	seq.clear();
+	return *this;
+}
+
 ${PREPEND_IMPLEMENTATION}
 
 ${SORT_IMPLEMENTATION}
@@ -627,7 +682,7 @@ ${STLT} ${CONTAINER}::evalchildren(int level) const
 	return s;
 }
 
-${STLT} * ${CONTAINER}::subschildren(const lst & ls, const lst & lr, bool no_pattern) const
+${STLT} * ${CONTAINER}::subschildren(const lst & ls, const lst & lr, unsigned options) const
 {
 	// returns a NULL pointer if nothing had to be substituted
 	// returns a pointer to a newly created epvector otherwise
@@ -635,7 +690,7 @@ ${STLT} * ${CONTAINER}::subschildren(const lst & ls, const lst & lr, bool no_pat
 
 	${STLT}::const_iterator cit = seq.begin(), end = seq.end();
 	while (cit != end) {
-		const ex & subsed_ex = cit->subs(ls, lr, no_pattern);
+		const ex & subsed_ex = cit->subs(ls, lr, options);
 		if (!are_ex_trivially_equal(*cit, subsed_ex)) {
 
 			// something changed, copy seq, subs and return it
@@ -655,7 +710,12 @@ ${STLT} * ${CONTAINER}::subschildren(const lst & ls, const lst & lr, bool no_pat
 
 			// copy rest
 			while (cit2 != end) {
-				s->push_back(cit2->subs(ls, lr, no_pattern));
+				try {
+					s->push_back(cit2->subs(ls, lr, options));
+				} catch (...) {
+					delete s;
+					throw;
+				}
 				++cit2;
 			}
 			return s;
