@@ -59,13 +59,6 @@ void ex::print(const print_context & c, unsigned level) const
 	bp->print(c, level);
 }
 
-/** Print expression to stream in a tree-like format suitable for debugging. */
-void ex::printtree(std::ostream & os) const
-{
-	GINAC_ASSERT(bp!=0);
-	bp->print(print_tree(os));
-}
-
 /** Little wrapper arount print to be called within a debugger. */
 void ex::dbgprint(void) const
 {
@@ -124,36 +117,38 @@ bool ex::find(const ex & pattern, lst & found) const
 		return true;
 	}
 	bool any_found = false;
-	for (unsigned i=0; i<nops(); i++)
+	for (size_t i=0; i<nops(); i++)
 		if (op(i).find(pattern, found))
 			any_found = true;
 	return any_found;
 }
 
-ex ex::operator[](const ex & index) const
-{
-	GINAC_ASSERT(bp!=0);
-	return (*bp)[index];
-}
-
-ex ex::operator[](int i) const
-{
-	GINAC_ASSERT(bp!=0);
-	return (*bp)[i];
-}
-
 /** Return modifyable operand/member at position i. */
-ex & ex::let_op(int i)
+ex & ex::let_op(size_t i)
 {
 	makewriteable();
 	GINAC_ASSERT(bp!=0);
 	return bp->let_op(i);
 }
 
+ex & ex::operator[](const ex & index)
+{
+	makewriteable();
+	GINAC_ASSERT(bp!=0);
+	return (*bp)[index];
+}
+
+ex & ex::operator[](size_t i)
+{
+	makewriteable();
+	GINAC_ASSERT(bp!=0);
+	return (*bp)[i];
+}
+
 /** Left hand side of relational expression. */
 ex ex::lhs(void) const
 {
-	if (!is_ex_of_type(*this,relational))
+	if (!is_a<relational>(*this))
 		throw std::runtime_error("ex::lhs(): not a relation");
 	return (*static_cast<relational *>(bp)).lhs();
 }
@@ -161,7 +156,7 @@ ex ex::lhs(void) const
 /** Right hand side of relational expression. */
 ex ex::rhs(void) const
 {
-	if (!is_ex_of_type(*this,relational))
+	if (!is_a<relational>(*this))
 		throw std::runtime_error("ex::rhs(): not a relation");
 	return (*static_cast<relational *>(bp)).rhs();
 }
