@@ -27,7 +27,6 @@
 #include <functional>
 
 #include "basic.h"
-#include "operators.h"
 
 namespace GiNaC {
 
@@ -68,12 +67,10 @@ class scalar_products;
 class ex
 {
 	friend class archive_node;
-	friend bool are_ex_trivially_equal(const ex &, const ex &);
-	template<class T> friend const T &ex_to(const ex &);
-	template<class T> friend bool is_a(const ex &);
-	template<class T> friend bool is_exactly_a(const ex &);
-	
-// member functions
+	friend inline bool are_ex_trivially_equal(const ex &, const ex &);
+	template<class T> friend inline const T &ex_to(const ex &);
+	template<class T> friend inline bool is_a(const ex &);
+	template<class T> friend inline bool is_exactly_a(const ex &);
 	
 	// default ctor, dtor, copy ctor, assignment operator and helpers
 public:
@@ -112,11 +109,10 @@ public:
 	}
 
 	void print(const print_context & c, unsigned level = 0) const;
-	void printtree(std::ostream & os) const;
 	void dbgprint(void) const;
 	void dbgprinttree(void) const;
 	bool info(unsigned inf) const { return bp->info(inf); }
-	unsigned nops() const { return bp->nops(); }
+	size_t nops() const { return bp->nops(); }
 	ex expand(unsigned options=0) const;
 	bool has(const ex & pattern) const { return bp->has(pattern); }
 	ex map(map_function & f) const { return bp->map(f); }
@@ -136,7 +132,8 @@ public:
 	ex primpart(const symbol &x) const;
 	ex primpart(const symbol &x, const ex &cont) const;
 	ex normal(int level = 0) const;
-	ex to_rational(lst &repl_lst) const { return bp->to_rational(repl_lst); }
+	ex to_rational(lst &repl_lst) const;
+	ex to_polynomial(lst &repl_lst) const;
 	ex smod(const numeric &xi) const { return bp->smod(xi); }
 	numeric max_coefficient(void) const;
 	ex collect(const ex & s, bool distributed = false) const { return bp->collect(s, distributed); }
@@ -147,8 +144,8 @@ public:
 	ex series(const ex & r, int order, unsigned options = 0) const;
 	bool match(const ex & pattern) const;
 	bool match(const ex & pattern, lst & repl_lst) const { return bp->match(pattern, repl_lst); }
-	ex subs(const lst & ls, const lst & lr, bool no_pattern = false) const { return bp->subs(ls, lr, no_pattern); }
-	ex subs(const ex & e, bool no_pattern = false) const { return bp->subs(e, no_pattern); }
+	ex subs(const lst & ls, const lst & lr, unsigned options = 0) const { return bp->subs(ls, lr, options); }
+	ex subs(const ex & e, unsigned options = 0) const { return bp->subs(e, options); }
 	exvector get_free_indices(void) const { return bp->get_free_indices(); }
 	ex simplify_indexed(void) const;
 	ex simplify_indexed(const scalar_products & sp) const;
@@ -158,11 +155,13 @@ public:
 	ex antisymmetrize(const lst & l) const;
 	ex symmetrize_cyclic(void) const;
 	ex symmetrize_cyclic(const lst & l) const;
-	ex simplify_ncmul(const exvector & v) const { return bp->simplify_ncmul(v); }
-	ex operator[](const ex & index) const;
-	ex operator[](int i) const;
-	ex op(int i) const { return bp->op(i); }
-	ex & let_op(int i);
+	ex eval_ncmul(const exvector & v) const { return bp->eval_ncmul(v); }
+	ex op(size_t i) const { return bp->op(i); }
+	ex operator[](const ex & index) const { return (*bp)[index]; }
+	ex operator[](size_t i) const { return (*bp)[i]; }
+	ex & let_op(size_t i);
+	ex & operator[](const ex & index);
+	ex & operator[](size_t i);
 	ex lhs(void) const;
 	ex rhs(void) const;
 	int compare(const ex & other) const;
@@ -209,9 +208,10 @@ protected:
 
 // member variables
 
-public:
-	basic *bp;      ///< pointer to basic object managed by this, direct manipulation deprecated
+private:
+	basic *bp;      ///< pointer to basic object managed by this
 #ifdef OBSCURE_CINT_HACK
+public:
 	static basic * last_created_or_assigned_bp;
 	static basic * dummy_bp;
 	static long last_created_or_assigned_exp;
@@ -368,7 +368,7 @@ inline bool are_ex_trivially_equal(const ex &e1, const ex &e2)
 }
 
 // wrapper functions around member functions
-inline unsigned nops(const ex & thisex)
+inline size_t nops(const ex & thisex)
 { return thisex.nops(); }
 
 inline ex expand(const ex & thisex, unsigned options = 0)
@@ -404,6 +404,9 @@ inline ex normal(const ex & thisex, int level=0)
 inline ex to_rational(const ex & thisex, lst & repl_lst)
 { return thisex.to_rational(repl_lst); }
 
+inline ex to_polynomial(const ex & thisex, lst & repl_lst)
+{ return thisex.to_polynomial(repl_lst); }
+
 inline ex collect(const ex & thisex, const ex & s, bool distributed = false)
 { return thisex.collect(s, distributed); }
 
@@ -425,11 +428,11 @@ inline ex series(const ex & thisex, const ex & r, int order, unsigned options =
 inline bool match(const ex & thisex, const ex & pattern, lst & repl_lst)
 { return thisex.match(pattern, repl_lst); }
 
-inline ex subs(const ex & thisex, const ex & e)
-{ return thisex.subs(e); }
+inline ex subs(const ex & thisex, const ex & e, unsigned options = 0)
+{ return thisex.subs(e, options); }
 
-inline ex subs(const ex & thisex, const lst & ls, const lst & lr)
-{ return thisex.subs(ls, lr); }
+inline ex subs(const ex & thisex, const lst & ls, const lst & lr, unsigned options = 0)
+{ return thisex.subs(ls, lr, options); }
 
 inline ex simplify_indexed(const ex & thisex)
 { return thisex.simplify_indexed(); }
@@ -455,7 +458,7 @@ inline ex symmetrize_cyclic(const ex & thisex)
 inline ex symmetrize_cyclic(const ex & thisex, const lst & l)
 { return thisex.symmetrize_cyclic(l); }
 
-inline ex op(const ex & thisex, int i)
+inline ex op(const ex & thisex, size_t i)
 { return thisex.op(i); }
 
 inline ex lhs(const ex & thisex)
@@ -470,11 +473,6 @@ inline bool is_zero(const ex & thisex)
 inline void swap(ex & e1, ex & e2)
 { e1.swap(e2); }
 
-// This makes STL algorithms use the more efficient swap operation for ex objects
-inline void iter_swap(std::vector<ex>::iterator i1, std::vector<ex>::iterator i2)
-{ i1->swap(*i2); }
-
-
 /* Function objects for STL sort() etc. */
 struct ex_is_less : public std::binary_function<ex, ex, bool> {
 	bool operator() (const ex &lh, const ex &rh) const { return lh.compare(rh) < 0; }
@@ -541,7 +539,57 @@ inline ex ex::map(ex (*f)(const ex & e)) const
 	return bp->map(fcn);
 }
 
+// convenience type checker template functions
+
+/** Check if ex is a handle to a T, including base classes. */
+template <class T>
+inline bool is_a(const ex &obj)
+{
+	return is_a<T>(*obj.bp);
+}
+
+/** Check if ex is a handle to a T, not including base classes. */
+template <class T>
+inline bool is_exactly_a(const ex &obj)
+{
+	return is_exactly_a<T>(*obj.bp);
+}
+
+/** Return a reference to the basic-derived class T object embedded in an
+ *  expression.  This is fast but unsafe: the result is undefined if the
+ *  expression does not contain a T object at its top level.  Hence, you
+ *  should generally check the type of e first.
+ *
+ *  @param e expression
+ *  @return reference to object of class T
+ *  @see is_exactly_a<class T>() */
+template <class T>
+inline const T &ex_to(const ex &e)
+{
+	GINAC_ASSERT(is_a<T>(e));
+	return static_cast<const T &>(*e.bp);
+}
 
 } // namespace GiNaC
 
+
+// Specializations of Standard Library algorithms
+namespace std {
+
+/** Specialization of std::swap() for ex objects. */
+template <>
+inline void swap(GiNaC::ex &a, GiNaC::ex &b)
+{
+	a.swap(b);
+}
+
+/** Specialization of std::iter_swap() for vector<ex> iterators. */
+template <>
+inline void iter_swap(vector<GiNaC::ex>::iterator i1, vector<GiNaC::ex>::iterator i2)
+{
+	i1->swap(*i2);
+}
+
+} // namespace std
+
 #endif // ndef __GINAC_EX_H__
