@@ -89,14 +89,14 @@ public:
 	void print(const print_context & c, unsigned level = 0) const;
 	unsigned precedence(void) const {return 10;}
 	bool info(unsigned inf) const;
-	unsigned nops() const;
-	ex op(int i) const;
-	ex & let_op(int i);
+	size_t nops() const;
+	ex op(size_t i) const;
 	ex map(map_function & f) const;
 	ex eval(int level=0) const;
 	ex to_rational(lst &repl_lst) const;
+	ex to_polynomial(lst &repl_lst) const;
 	bool match(const ex & pattern, lst & repl_lst) const;
-	ex subs(const lst & ls, const lst & lr, bool no_pattern = false) const;
+	ex subs(const lst & ls, const lst & lr, unsigned options = 0) const;
 protected:
 	int compare_same_type(const basic & other) const;
 	bool is_equal_same_type(const basic & other) const;
@@ -163,7 +163,7 @@ protected:
 	bool is_canonical() const;
 	epvector * expandchildren(unsigned options) const;
 	epvector * evalchildren(int level) const;
-	epvector * subschildren(const lst & ls, const lst & lr, bool no_pattern = false) const;
+	epvector * subschildren(const lst & ls, const lst & lr, unsigned options = 0) const;
 	
 // member variables
 	
