@@ -39,10 +39,9 @@ sub generate {
 $declare_function_macro = generate(
 	<<'END_OF_DECLARE_FUNCTION_MACRO','typename T${N}','const T${N} & p${N}','GiNaC::ex(p${N})');
 #define DECLARE_FUNCTION_${N}P(NAME) \\
-extern const unsigned function_index_##NAME; \\
-template<${SEQ1}> \\
-inline const GiNaC::function NAME(${SEQ2}) { \\
-	return GiNaC::function(function_index_##NAME, ${SEQ3}); \\
+class NAME##_SERIAL { public: static unsigned serial; }; \\
+template<${SEQ1}> const GiNaC::function NAME(${SEQ2}) { \\
+	return GiNaC::function(NAME##_SERIAL::serial, ${SEQ3}); \\
 }
 
 END_OF_DECLARE_FUNCTION_MACRO
@@ -197,7 +196,7 @@ $declare_function_macro
 // end of generated lines
 
 #define REGISTER_FUNCTION(NAME,OPT) \\
-const unsigned function_index_##NAME= \\
+unsigned NAME##_SERIAL::serial = \\
 	GiNaC::function::register_new(GiNaC::function_options(#NAME).OPT);
 
 namespace GiNaC {
@@ -224,6 +223,7 @@ typedef ex (* evalf_funcp_exvector)(const exvector &);
 typedef ex (* derivative_funcp_exvector)(const exvector &, unsigned);
 typedef ex (* series_funcp_exvector)(const exvector &, const relational &, int, unsigned);
 
+
 class function_options
 {
 	friend class function;
@@ -289,6 +289,12 @@ protected:
 	ex symtree;
 };
 
+
+/** Exception class thrown by classes which provide their own series expansion
+ *  to signal that ordinary Taylor expansion is safe. */
+class do_taylor {};
+
+
 /** The class function is used to implement builtin functions like sin, cos...
 	and user defined functions */
 class function : public exprseq
@@ -364,8 +370,15 @@ template<> inline bool is_exactly_a<function>(const basic & obj)
 	return obj.tinfo()==TINFO_function;
 }
 
-#define is_ex_the_function(OBJ, FUNCNAME) \\
-	(GiNaC::is_exactly_a<GiNaC::function>(OBJ) && GiNaC::ex_to<GiNaC::function>(OBJ).get_serial() == function_index_##FUNCNAME)
+template <typename T>
+inline bool is_the_function(const ex & x)
+{
+	return is_exactly_a<function>(x)
+	    && ex_to<function>(x).get_serial() == T::serial;
+}
+
+// Check whether OBJ is the specified symbolic function.
+#define is_ex_the_function(OBJ, FUNCNAME) (GiNaC::is_the_function<FUNCNAME##_SERIAL>(OBJ))
 
 } // namespace GiNaC
 
@@ -406,6 +419,7 @@ $implementation=<<END_OF_IMPLEMENTATION;
 #include <list>
 
 #include "function.h"
+#include "operators.h"
 #include "fderivative.h"
 #include "ex.h"
 #include "lst.h"
@@ -549,7 +563,7 @@ void function_options::test_and_set_nparams(unsigned n)
 		// we do not throw an exception here because this code is
 		// usually executed before main(), so the exception could not
 		// caught anyhow
-		std::cerr << "WARNING: number of parameters ("
+		std::cerr << "WARNING: " << name << "(): number of parameters ("
 		          << n << ") differs from number set before (" 
 		          << nparams << ")" << std::endl;
 	}
@@ -627,7 +641,7 @@ function::function(unsigned ser, exvector * vp)
 //////////
 
 /** Construct object from archive_node. */
-function::function(const archive_node &n, const lst &sym_lst) : inherited(n, sym_lst)
+function::function(const archive_node &n, lst &sym_lst) : inherited(n, sym_lst)
 {
 	// Find serial number by function name
 	std::string s;
@@ -647,7 +661,7 @@ function::function(const archive_node &n, const lst &sym_lst) : inherited(n, sym
 }
 
 /** Unarchive the object. */
-ex function::unarchive(const archive_node &n, const lst &sym_lst)
+ex function::unarchive(const archive_node &n, lst &sym_lst)
 {
 	return (new function(n, sym_lst))->setflag(status_flags::dynallocated);
 }
@@ -670,7 +684,7 @@ void function::print(const print_context & c, unsigned level) const
 {
 	GINAC_ASSERT(serial<registered_functions().size());
 
-	if (is_of_type(c, print_tree)) {
+	if (is_a<print_tree>(c)) {
 
 		c.s << std::string(level, ' ') << class_name() << " "
 		    << registered_functions()[serial].name
@@ -678,16 +692,16 @@ void function::print(const print_context & c, unsigned level) const
 		    << ", nops=" << nops()
 		    << std::endl;
 		unsigned delta_indent = static_cast<const print_tree &>(c).delta_indent;
-		for (unsigned i=0; i<seq.size(); ++i)
+		for (size_t i=0; i<seq.size(); ++i)
 			seq[i].print(c, level + delta_indent);
 		c.s << std::string(level + delta_indent, ' ') << "=====" << std::endl;
 
-	} else if (is_of_type(c, print_csrc)) {
+	} else if (is_a<print_csrc>(c)) {
 
 		// Print function name in lowercase
 		std::string lname = registered_functions()[serial].name;
-		unsigned num = lname.size();
-		for (unsigned i=0; i<num; i++)
+		size_t num = lname.size();
+		for (size_t i=0; i<num; i++)
 			lname[i] = tolower(lname[i]);
 		c.s << lname << "(";
 
@@ -701,7 +715,7 @@ void function::print(const print_context & c, unsigned level) const
 		}
 		c.s << ")";
 
-	} else if (is_of_type(c, print_latex)) {
+	} else if (is_a<print_latex>(c)) {
 		c.s << registered_functions()[serial].TeX_name;
 		printseq(c, '(', ',', ')', exprseq::precedence(), function::precedence());
 	} else {
@@ -808,11 +822,11 @@ ${evalf_switch_statement}
 unsigned function::calchash(void) const
 {
 	unsigned v = golden_ratio_hash(golden_ratio_hash(tinfo()) ^ serial);
-	for (unsigned i=0; i<nops(); i++) {
-		v = rotate_left_31(v);
+	for (size_t i=0; i<nops(); i++) {
+		v = rotate_left(v);
 		v ^= this->op(i).gethash();
 	}
-	v &= 0x7FFFFFFFU;
+
 	if (flags & status_flags::evaluated) {
 		setflag(status_flags::hash_calculated);
 		hashvalue = v;
@@ -866,14 +880,14 @@ ex function::derivative(const symbol & s) const
 {
 	ex result;
 
-	if (serial == function_index_Order) {
+	if (serial == Order_SERIAL::serial) {
 		// Order Term function only differentiates the argument
 		return Order(seq[0].diff(s));
 	} else {
 		// Chain rule
 		ex arg_diff;
-		unsigned num = seq.size();
-		for (unsigned i=0; i<num; i++) {
+		size_t num = seq.size();
+		for (size_t i=0; i<num; i++) {
 			arg_diff = seq[i].diff(s);
 			// We apply the chain rule only when it makes sense.  This is not
 			// just for performance reasons but also to allow functions to
@@ -889,7 +903,7 @@ ex function::derivative(const symbol & s) const
 
 int function::compare_same_type(const basic & other) const
 {
-	GINAC_ASSERT(is_of_type(other, function));
+	GINAC_ASSERT(is_a<function>(other));
 	const function & o = static_cast<const function &>(other);
 
 	if (serial != o.serial)
@@ -900,7 +914,7 @@ int function::compare_same_type(const basic & other) const
 
 bool function::is_equal_same_type(const basic & other) const
 {
-	GINAC_ASSERT(is_of_type(other, function));
+	GINAC_ASSERT(is_a<function>(other));
 	const function & o = static_cast<const function &>(other);
 
 	if (serial != o.serial)
@@ -911,7 +925,7 @@ bool function::is_equal_same_type(const basic & other) const
 
 bool function::match_same_type(const basic & other) const
 {
-	GINAC_ASSERT(is_of_type(other, function));
+	GINAC_ASSERT(is_a<function>(other));
 	const function & o = static_cast<const function &>(other);
 
 	return serial == o.serial;
@@ -990,20 +1004,20 @@ std::vector<function_options> & function::registered_functions(void)
 
 bool function::lookup_remember_table(ex & result) const
 {
-	return remember_table::remember_tables()[serial].lookup_entry(*this,result);
+	return remember_table::remember_tables()[this->serial].lookup_entry(*this,result);
 }
 
 void function::store_remember_table(ex const & result) const
 {
-	remember_table::remember_tables()[serial].add_entry(*this,result);
+	remember_table::remember_tables()[this->serial].add_entry(*this,result);
 }
 
 // public
 
 unsigned function::register_new(function_options const & opt)
 {
-	unsigned same_name = 0;
-	for (unsigned i=0; i<registered_functions().size(); ++i) {
+	size_t same_name = 0;
+	for (size_t i=0; i<registered_functions().size(); ++i) {
 		if (registered_functions()[i].name==opt.name) {
 			++same_name;
 		}
