@@ -30,10 +30,12 @@
 #include <stdexcept>
 #include <string>
 #include <sstream>
+#include <limits>
 
 #include "numeric.h"
 #include "ex.h"
 #include "print.h"
+#include "operators.h"
 #include "archive.h"
 #include "tostring.h"
 #include "utils.h"
@@ -243,7 +245,7 @@ numeric::numeric(const cln::cl_N &z) : basic(TINFO_numeric)
 // archiving
 //////////
 
-numeric::numeric(const archive_node &n, const lst &sym_lst) : inherited(n, sym_lst)
+numeric::numeric(const archive_node &n, lst &sym_lst) : inherited(n, sym_lst)
 {
 	cln::cl_N ctorval = 0;
 
@@ -456,9 +458,9 @@ void numeric::print(const print_context & c, unsigned level) const
 
 		// Set precision
 		if (is_a<print_csrc_double>(c))
-			c.s.precision(16);
+			c.s.precision(std::numeric_limits<double>::digits10 + 1);
 		else
-			c.s.precision(7);
+			c.s.precision(std::numeric_limits<float>::digits10 + 1);
 
 		if (this->is_real()) {
 
@@ -618,7 +620,7 @@ ex numeric::coeff(const ex & s, int n) const
  *  sign as a multiplicative factor. */
 bool numeric::has(const ex &other) const
 {
-	if (!is_ex_exactly_of_type(other, numeric))
+	if (!is_exactly_a<numeric>(other))
 		return false;
 	const numeric &o = ex_to<numeric>(other);
 	if (this->is_equal(o) || this->is_equal(-o))
@@ -682,11 +684,13 @@ bool numeric::is_equal_same_type(const basic &other) const
 
 unsigned numeric::calchash(void) const
 {
-	// Use CLN's hashcode.  Warning: It depends only on the number's value, not
-	// its type or precision (i.e. a true equivalence relation on numbers).  As
-	// a consequence, 3 and 3.0 share the same hashvalue.
+	// Base computation of hashvalue on CLN's hashcode.  Note: That depends
+	// only on the number's value, not its type or precision (i.e. a true
+	// equivalence relation on numbers).  As a consequence, 3 and 3.0 share
+	// the same hashvalue.  That shouldn't really matter, though.
 	setflag(status_flags::hash_calculated);
-	return (hashvalue = cln::equal_hashcode(cln::the<cln::cl_N>(value)) | 0x80000000U);
+	hashvalue = golden_ratio_hash(cln::equal_hashcode(cln::the<cln::cl_N>(value)));
+	return hashvalue;
 }
 
 
@@ -754,8 +758,9 @@ const numeric numeric::div(const numeric &other) const
  *  returns result as a numeric object. */
 const numeric numeric::power(const numeric &other) const
 {
-	// Efficiency shortcut: trap the neutral exponent by pointer.
-	if (&other==_num1_p)
+	// Shortcut for efficiency and numeric stability (as in 1.0 exponent):
+	// trap the neutral exponent.
+	if (&other==_num1_p || cln::equal(cln::the<cln::cl_N>(other.value),cln::the<cln::cl_N>(_num1.value)))
 		return *this;
 	
 	if (cln::zerop(cln::the<cln::cl_N>(value))) {
@@ -772,26 +777,39 @@ const numeric numeric::power(const numeric &other) const
 }
 
 
+
+/** Numerical addition method.  Adds argument to *this and returns result as
+ *  a numeric object on the heap.  Use internally only for direct wrapping into
+ *  an ex object, where the result would end up on the heap anyways. */
 const numeric &numeric::add_dyn(const numeric &other) const
 {
-	// Efficiency shortcut: trap the neutral element by pointer.
+	// Efficiency shortcut: trap the neutral element by pointer.  This hack
+	// is supposed to keep the number of distinct numeric objects low.
 	if (this==_num0_p)
 		return other;
 	else if (&other==_num0_p)
 		return *this;
 	
 	return static_cast<const numeric &>((new numeric(cln::the<cln::cl_N>(value)+cln::the<cln::cl_N>(other.value)))->
-										setflag(status_flags::dynallocated));
+	                                    setflag(status_flags::dynallocated));
 }
 
 
+/** Numerical subtraction method.  Subtracts argument from *this and returns
+ *  result as a numeric object on the heap.  Use internally only for direct
+ *  wrapping into an ex object, where the result would end up on the heap
+ *  anyways. */
 const numeric &numeric::sub_dyn(const numeric &other) const
 {
 	return static_cast<const numeric &>((new numeric(cln::the<cln::cl_N>(value)-cln::the<cln::cl_N>(other.value)))->
-										setflag(status_flags::dynallocated));
+	                                    setflag(status_flags::dynallocated));
 }
 
 
+/** Numerical multiplication method.  Multiplies *this and argument and returns
+ *  result as a numeric object on the heap.  Use internally only for direct
+ *  wrapping into an ex object, where the result would end up on the heap
+ *  anyways. */
 const numeric &numeric::mul_dyn(const numeric &other) const
 {
 	// Efficiency shortcut: trap the neutral element by pointer.
@@ -801,23 +819,35 @@ const numeric &numeric::mul_dyn(const numeric &other) const
 		return *this;
 	
 	return static_cast<const numeric &>((new numeric(cln::the<cln::cl_N>(value)*cln::the<cln::cl_N>(other.value)))->
-										setflag(status_flags::dynallocated));
+	                                    setflag(status_flags::dynallocated));
 }
 
 
+/** Numerical division method.  Divides *this by argument and returns result as
+ *  a numeric object on the heap.  Use internally only for direct wrapping
+ *  into an ex object, where the result would end up on the heap
+ *  anyways.
+ *
+ *  @exception overflow_error (division by zero) */
 const numeric &numeric::div_dyn(const numeric &other) const
 {
 	if (cln::zerop(cln::the<cln::cl_N>(other.value)))
 		throw std::overflow_error("division by zero");
 	return static_cast<const numeric &>((new numeric(cln::the<cln::cl_N>(value)/cln::the<cln::cl_N>(other.value)))->
-										setflag(status_flags::dynallocated));
+	                                    setflag(status_flags::dynallocated));
 }
 
 
+/** Numerical exponentiation.  Raises *this to the power given as argument and
+ *  returns result as a numeric object on the heap.  Use internally only for
+ *  direct wrapping into an ex object, where the result would end up on the
+ *  heap anyways. */
 const numeric &numeric::power_dyn(const numeric &other) const
 {
-	// Efficiency shortcut: trap the neutral exponent by pointer.
-	if (&other==_num1_p)
+	// Efficiency shortcut: trap the neutral exponent (first try by pointer, then
+	// try harder, since calls to cln::expt() below may return amazing results for
+	// floating point exponent 1.0).
+	if (&other==_num1_p || cln::equal(cln::the<cln::cl_N>(other.value),cln::the<cln::cl_N>(_num1.value)))
 		return *this;
 	
 	if (cln::zerop(cln::the<cln::cl_N>(value))) {
@@ -945,7 +975,7 @@ bool numeric::is_zero(void) const
 /** True if object is not complex and greater than zero. */
 bool numeric::is_positive(void) const
 {
-	if (this->is_real())
+	if (cln::instanceof(value, cln::cl_R_ring))  // real?
 		return cln::plusp(cln::the<cln::cl_R>(value));
 	return false;
 }
@@ -954,7 +984,7 @@ bool numeric::is_positive(void) const
 /** True if object is not complex and less than zero. */
 bool numeric::is_negative(void) const
 {
-	if (this->is_real())
+	if (cln::instanceof(value, cln::cl_R_ring))  // real?
 		return cln::minusp(cln::the<cln::cl_R>(value));
 	return false;
 }
@@ -970,28 +1000,28 @@ bool numeric::is_integer(void) const
 /** True if object is an exact integer greater than zero. */
 bool numeric::is_pos_integer(void) const
 {
-	return (this->is_integer() && cln::plusp(cln::the<cln::cl_I>(value)));
+	return (cln::instanceof(value, cln::cl_I_ring) && cln::plusp(cln::the<cln::cl_I>(value)));
 }
 
 
 /** True if object is an exact integer greater or equal zero. */
 bool numeric::is_nonneg_integer(void) const
 {
-	return (this->is_integer() && !cln::minusp(cln::the<cln::cl_I>(value)));
+	return (cln::instanceof(value, cln::cl_I_ring) && !cln::minusp(cln::the<cln::cl_I>(value)));
 }
 
 
 /** True if object is an exact even integer. */
 bool numeric::is_even(void) const
 {
-	return (this->is_integer() && cln::evenp(cln::the<cln::cl_I>(value)));
+	return (cln::instanceof(value, cln::cl_I_ring) && cln::evenp(cln::the<cln::cl_I>(value)));
 }
 
 
 /** True if object is an exact odd integer. */
 bool numeric::is_odd(void) const
 {
-	return (this->is_integer() && cln::oddp(cln::the<cln::cl_I>(value)));
+	return (cln::instanceof(value, cln::cl_I_ring) && cln::oddp(cln::the<cln::cl_I>(value)));
 }
 
 
@@ -1000,7 +1030,9 @@ bool numeric::is_odd(void) const
  *  @return  true if object is exact integer and prime. */
 bool numeric::is_prime(void) const
 {
-	return (this->is_integer() && cln::isprobprime(cln::the<cln::cl_I>(value)));
+	return (cln::instanceof(value, cln::cl_I_ring)  // integer?
+	     && cln::plusp(cln::the<cln::cl_I>(value))  // positive?
+	     && cln::isprobprime(cln::the<cln::cl_I>(value)));
 }
 
 
@@ -1163,8 +1195,8 @@ const numeric numeric::imag(void) const
  *  cases. */
 const numeric numeric::numer(void) const
 {
-	if (this->is_integer())
-		return numeric(*this);
+	if (cln::instanceof(value, cln::cl_I_ring))
+		return numeric(*this);  // integer case
 	
 	else if (cln::instanceof(value, cln::cl_RA_ring))
 		return numeric(cln::numerator(cln::the<cln::cl_RA>(value)));
@@ -1194,8 +1226,8 @@ const numeric numeric::numer(void) const
  *  (i.e denom(4/3+5/6*I) == 6), one in all other cases. */
 const numeric numeric::denom(void) const
 {
-	if (this->is_integer())
-		return _num1;
+	if (cln::instanceof(value, cln::cl_I_ring))
+		return _num1;  // integer case
 	
 	if (cln::instanceof(value, cln::cl_RA_ring))
 		return numeric(cln::denominator(cln::the<cln::cl_RA>(value)));
@@ -1225,7 +1257,7 @@ const numeric numeric::denom(void) const
  *  in two's complement if it is an integer, 0 otherwise. */    
 int numeric::int_length(void) const
 {
-	if (this->is_integer())
+	if (cln::instanceof(value, cln::cl_I_ring))
 		return cln::integer_length(cln::the<cln::cl_I>(value));
 	else
 		return 0;
@@ -1252,14 +1284,14 @@ const numeric exp(const numeric &x)
 
 /** Natural logarithm.
  *
- *  @param z complex number
+ *  @param x complex number
  *  @return  arbitrary precision numerical log(x).
  *  @exception pole_error("log(): logarithmic pole",0) */
-const numeric log(const numeric &z)
+const numeric log(const numeric &x)
 {
-	if (z.is_zero())
+	if (x.is_zero())
 		throw pole_error("log(): logarithmic pole",0);
-	return cln::log(z.to_cl_N());
+	return cln::log(x.to_cl_N());
 }
 
 
@@ -1310,8 +1342,8 @@ const numeric acos(const numeric &x)
 
 /** Arcustangent.
  *
- *  @param z complex number
- *  @return atan(z)
+ *  @param x complex number
+ *  @return atan(x)
  *  @exception pole_error("atan(): logarithmic pole",0) */
 const numeric atan(const numeric &x)
 {
@@ -1802,7 +1834,7 @@ const numeric irem(const numeric &a, const numeric &b)
 /** Numeric integer remainder.
  *  Equivalent to Maple's irem(a,b,'q') it obeyes the relation
  *  irem(a,b,q) == a - q*b.  In general, mod(a,b) has the sign of b or is zero,
- *  and irem(a,b) has the sign of a or is zero.  
+ *  and irem(a,b) has the sign of a or is zero.
  *
  *  @return remainder of a/b and quotient stored in q if both are integer,
  *  0 otherwise.
@@ -1892,16 +1924,16 @@ const numeric lcm(const numeric &a, const numeric &b)
 
 
 /** Numeric square root.
- *  If possible, sqrt(z) should respect squares of exact numbers, i.e. sqrt(4)
+ *  If possible, sqrt(x) should respect squares of exact numbers, i.e. sqrt(4)
  *  should return integer 2.
  *
- *  @param z numeric argument
- *  @return square root of z. Branch cut along negative real axis, the negative
- *  real axis itself where imag(z)==0 and real(z)<0 belongs to the upper part
- *  where imag(z)>0. */
-const numeric sqrt(const numeric &z)
+ *  @param x numeric argument
+ *  @return square root of x. Branch cut along negative real axis, the negative
+ *  real axis itself where imag(x)==0 and real(x)<0 belongs to the upper part
+ *  where imag(x)>0. */
+const numeric sqrt(const numeric &x)
 {
-	return cln::sqrt(z.to_cl_N());
+	return cln::sqrt(x.to_cl_N());
 }
 
 
