@@ -26,6 +26,8 @@
 #include "basic.h"
 #include "ex.h"
 
+#include <stdexcept>
+
 #include <cln/number.h>
 // forward decln of cln::cl_N, since cln/complex_class.h is not included:
 namespace cln { class cl_N; }
@@ -62,6 +64,17 @@ private:
 	static bool too_late;               ///< Already one object present
 };
 
+
+/** Exception class thrown when a singularity is encountered. */
+class pole_error : public std::domain_error {
+public:
+	explicit pole_error(const std::string& what_arg, int degree);
+	int degree() const;
+private:
+	int deg;
+};
+
+
 /** This class is a wrapper around CLN-numbers within the GiNaC class
  *  hierarchy. Objects of this type may directly be created by the user.*/
 class numeric : public basic
@@ -93,6 +106,7 @@ public:
 	ex evalf(int level = 0) const;
 	ex normal(lst &sym_lst, lst &repl_lst, int level = 0) const;
 	ex to_rational(lst &repl_lst) const;
+	ex to_polynomial(lst &repl_lst) const;
 	numeric integer_content(void) const;
 	ex smod(const numeric &xi) const;
 	numeric max_coefficient(void) const;
@@ -165,14 +179,12 @@ protected:
 	cln::cl_number value;
 };
 
+
 // global constants
 
 extern const numeric I;
 extern _numeric_digits Digits;
 
-// deprecated macro, for internal use only
-#define is_a_numeric_hash(x) ((x)&0x80000000U)
-
 // global functions
 
 const numeric exp(const numeric &x);
