@@ -27,48 +27,48 @@
 namespace GiNaC {
 
 print_context::print_context()
-	: s(std::cout) {}
-print_context::print_context(std::ostream & os)
-	: s(os) {}
+	: s(std::cout), options(0) {}
+print_context::print_context(std::ostream & os, unsigned opt)
+	: s(os), options(opt) {}
 
 print_latex::print_latex()
 	: print_context(std::cout) {}
-print_latex::print_latex(std::ostream & os)
-	: print_context(os) {}
+print_latex::print_latex(std::ostream & os, unsigned opt)
+	: print_context(os, opt) {}
 
 print_python::print_python()
 	: print_context(std::cout) {}
-print_python::print_python(std::ostream & os)
-	: print_context(os) {}
+print_python::print_python(std::ostream & os, unsigned opt)
+	: print_context(os, opt) {}
 
 print_python_repr::print_python_repr()
 	: print_context(std::cout) {}
-print_python_repr::print_python_repr(std::ostream & os)
-	: print_context(os) {}
+print_python_repr::print_python_repr(std::ostream & os, unsigned opt)
+	: print_context(os, opt) {}
 
 print_tree::print_tree(unsigned d)
 	: print_context(std::cout), delta_indent(d) {}
-print_tree::print_tree(std::ostream & os, unsigned d)
-	: print_context(os), delta_indent(d) {}
+print_tree::print_tree(std::ostream & os, unsigned opt, unsigned d)
+	: print_context(os, opt), delta_indent(d) {}
 
 print_csrc::print_csrc()
 	: print_context(std::cout) {}
-print_csrc::print_csrc(std::ostream & os)
-	: print_context(os) {}
+print_csrc::print_csrc(std::ostream & os, unsigned opt)
+	: print_context(os, opt) {}
 
 print_csrc_float::print_csrc_float()
 	: print_csrc(std::cout) {}
-print_csrc_float::print_csrc_float(std::ostream & os)
-	: print_csrc(os) {}
+print_csrc_float::print_csrc_float(std::ostream & os, unsigned opt)
+	: print_csrc(os, opt) {}
 
 print_csrc_double::print_csrc_double()
 	: print_csrc(std::cout) {}
-print_csrc_double::print_csrc_double(std::ostream & os)
-	: print_csrc(os) {}
+print_csrc_double::print_csrc_double(std::ostream & os, unsigned opt)
+	: print_csrc(os, opt) {}
 
 print_csrc_cl_N::print_csrc_cl_N()
 	: print_csrc(std::cout) {}
-print_csrc_cl_N::print_csrc_cl_N(std::ostream & os)
-	: print_csrc(os) {}
+print_csrc_cl_N::print_csrc_cl_N(std::ostream & os, unsigned opt)
+	: print_csrc(os, opt) {}
 
 } // namespace GiNaC
