@@ -31,6 +31,7 @@
 #include "mul.h"
 #include "power.h"
 #include "relational.h"
+#include "operators.h"
 #include "symbol.h"
 #include "print.h"
 #include "archive.h"
@@ -84,7 +85,7 @@ pseries::pseries(const ex &rel_, const epvector &ops_) : basic(TINFO_pseries), s
  *  Archiving
  */
 
-pseries::pseries(const archive_node &n, const lst &sym_lst) : inherited(n, sym_lst)
+pseries::pseries(const archive_node &n, lst &sym_lst) : inherited(n, sym_lst)
 {
 	for (unsigned int i=0; true; ++i) {
 		ex rest;
@@ -125,8 +126,8 @@ void pseries::print(const print_context & c, unsigned level) const
 		    << std::hex << ", hash=0x" << hashvalue << ", flags=0x" << flags << std::dec
 		    << std::endl;
 		unsigned delta_indent = static_cast<const print_tree &>(c).delta_indent;
-		unsigned num = seq.size();
-		for (unsigned i=0; i<num; ++i) {
+		size_t num = seq.size();
+		for (size_t i=0; i<num; ++i) {
 			seq[i].rest.print(c, level + delta_indent);
 			seq[i].coeff.print(c, level + delta_indent);
 			c.s << std::string(level + delta_indent, ' ') << "-----" << std::endl;
@@ -140,8 +141,8 @@ void pseries::print(const print_context & c, unsigned level) const
 		c.s << ',';
 		point.print(c);
 		c.s << "),[";
-		unsigned num = seq.size();
-		for (unsigned i=0; i<num; ++i) {
+		size_t num = seq.size();
+		for (size_t i=0; i<num; ++i) {
 			if (i)
 				c.s << ',';
 			c.s << '(';
@@ -253,22 +254,18 @@ int pseries::compare_same_type(const basic & other) const
 }
 
 /** Return the number of operands including a possible order term. */
-unsigned pseries::nops(void) const
+size_t pseries::nops(void) const
 {
 	return seq.size();
 }
 
 /** Return the ith term in the series when represented as a sum. */
-ex pseries::op(int i) const
+ex pseries::op(size_t i) const
 {
-	if (i < 0 || unsigned(i) >= seq.size())
+	if (i >= seq.size())
 		throw (std::out_of_range("op() out of range"));
-	return seq[i].rest * power(var - point, seq[i].coeff);
-}
 
-ex &pseries::let_op(int i)
-{
-	throw (std::logic_error("let_op not defined for pseries"));
+	return seq[i].rest * power(var - point, seq[i].coeff);
 }
 
 /** Return degree of highest power of the series.  This is usually the exponent
@@ -409,13 +406,13 @@ ex pseries::evalf(int level) const
 	return (new pseries(relational(var,point), new_seq))->setflag(status_flags::dynallocated | status_flags::evaluated);
 }
 
-ex pseries::subs(const lst & ls, const lst & lr, bool no_pattern) const
+ex pseries::subs(const lst & ls, const lst & lr, unsigned options) const
 {
 	// If expansion variable is being substituted, convert the series to a
 	// polynomial and do the substitution there because the result might
 	// no longer be a power series
 	if (ls.has(var))
-		return convert_to_poly(true).subs(ls, lr, no_pattern);
+		return convert_to_poly(true).subs(ls, lr, options);
 	
 	// Otherwise construct a new series with substituted coefficients and
 	// expansion point
@@ -423,10 +420,10 @@ ex pseries::subs(const lst & ls, const lst & lr, bool no_pattern) const
 	newseq.reserve(seq.size());
 	epvector::const_iterator it = seq.begin(), itend = seq.end();
 	while (it != itend) {
-		newseq.push_back(expair(it->rest.subs(ls, lr, no_pattern), it->coeff));
+		newseq.push_back(expair(it->rest.subs(ls, lr, options), it->coeff));
 		++it;
 	}
-	return (new pseries(relational(var,point.subs(ls, lr, no_pattern)), newseq))->setflag(status_flags::dynallocated);
+	return (new pseries(relational(var,point.subs(ls, lr, options)), newseq))->setflag(status_flags::dynallocated);
 }
 
 /** Implementation of ex::expand() for a power series.  It expands all the
@@ -516,7 +513,7 @@ ex basic::series(const relational & r, int order, unsigned options) const
 	epvector seq;
 	numeric fac = 1;
 	ex deriv = *this;
-	ex coeff = deriv.subs(r);
+	ex coeff = deriv.subs(r, subs_options::no_pattern);
 	const symbol &s = ex_to<symbol>(r.lhs());
 	
 	if (!coeff.is_zero())
@@ -532,7 +529,7 @@ ex basic::series(const relational & r, int order, unsigned options) const
 		if (deriv.is_zero())  // Series terminates
 			return pseries(r, seq);
 
-		coeff = deriv.subs(r);
+		coeff = deriv.subs(r, subs_options::no_pattern);
 		if (!coeff.is_zero())
 			seq.push_back(expair(fac.inverse() * coeff, n));
 	}
@@ -874,7 +871,7 @@ ex power::series(const relational & r, int order, unsigned options) const
 	// Basis is not a series, may there be a singularity?
 	bool must_expand_basis = false;
 	try {
-		basis.subs(r);
+		basis.subs(r, subs_options::no_pattern);
 	} catch (pole_error) {
 		must_expand_basis = true;
 	}
@@ -884,7 +881,7 @@ ex power::series(const relational & r, int order, unsigned options) const
 		return basic::series(r, order, options);
 		
 	// Is the expression of type 0^something?
-	if (!must_expand_basis && !basis.subs(r).is_zero())
+	if (!must_expand_basis && !basis.subs(r, subs_options::no_pattern).is_zero())
 		return basic::series(r, order, options);
 
 	// Singularity encountered, is the basis equal to (var - point)?
@@ -947,7 +944,7 @@ ex ex::series(const ex & r, int order, unsigned options) const
 	ex e;
 	relational rel_;
 	
-	if (is_exactly_a<relational>(r))
+	if (is_a<relational>(r))
 		rel_ = ex_to<relational>(r);
 	else if (is_a<symbol>(r))
 		rel_ = relational(r,_ex0);
