@@ -1,4 +1,4 @@
-/* A Bison parser, made from /home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy
+/* A Bison parser, made from /home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy
    by GNU bison 1.35.  */
 
 #define YYBISON 1  /* Identify Bison output.  */
@@ -27,7 +27,7 @@
 # define	T_SCORE	278
 # define	NEG	279
 
-#line 29 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 29 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 
 #include "config.h"
 
@@ -52,7 +52,7 @@ static char *orig_basic_word_break_characters;
 static const char *orig_basic_word_break_characters;
 #endif
 
-// Expression stack for ", "" and """
+// Expression stack for %, %% and %%%
 static void push(const ex &e);
 static ex exstack[3];
 
@@ -1064,7 +1064,7 @@ yyreduce:
   switch (yyn) {
 
 case 4:
-#line 121 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 121 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {
 		try {
 			cout << yyvsp[-1] << endl;
@@ -1076,7 +1076,7 @@ case 4:
 	}
     break;
 case 5:
-#line 130 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 130 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {
 		try {
 			push(yyvsp[-1]);
@@ -1087,7 +1087,7 @@ case 5:
 	}
     break;
 case 6:
-#line 138 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 138 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {
 		try {
 			yyvsp[-2].print(print_tree(std::cout));
@@ -1098,7 +1098,7 @@ case 6:
 	}
     break;
 case 7:
-#line 146 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 146 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {
 		try {
 			ex e = yyvsp[-2];
@@ -1115,7 +1115,7 @@ case 7:
 	}
     break;
 case 8:
-#line 160 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 160 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {
 		try {
 			yyvsp[-2].print(print_latex(std::cout)); cout << endl;
@@ -1126,7 +1126,7 @@ case 8:
 	}
     break;
 case 9:
-#line 168 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 168 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {
 		try {
 			yyvsp[-2].print(print_csrc_double(std::cout)); cout << endl;
@@ -1137,39 +1137,39 @@ case 9:
 	}
     break;
 case 10:
-#line 176 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 176 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {print_help(ex_to<symbol>(yyvsp[0]).get_name());}
     break;
 case 11:
-#line 177 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 177 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {print_help("time");}
     break;
 case 12:
-#line 178 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 178 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {print_help("print");}
     break;
 case 13:
-#line 179 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 179 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {print_help("iprint");}
     break;
 case 14:
-#line 180 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 180 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {print_help("print_latex");}
     break;
 case 15:
-#line 181 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 181 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {print_help("print_csrc");}
     break;
 case 16:
-#line 182 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 182 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {print_help_topics();}
     break;
 case 17:
-#line 183 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 183 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {YYACCEPT;}
     break;
 case 18:
-#line 184 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 184 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {
 		cout << "This program is free software; you can redistribute it and/or modify it under\n";
 		cout << "the terms of the GNU General Public License as published by the Free Software\n";
@@ -1185,19 +1185,19 @@ case 18:
 	}
     break;
 case 19:
-#line 197 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 197 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {cout << "Nothing happens.\n";}
     break;
 case 20:
-#line 198 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 198 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {cout << "You're not carrying anything.\n";}
     break;
 case 21:
-#line 199 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 199 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {cout << "You're in a twisty little maze of passages, all alike.\n";}
     break;
 case 22:
-#line 200 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 200 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {
 		cout << "If you were to quit now, you would score ";
 		cout << (syms.size() > 350 ? 350 : syms.size());
@@ -1205,11 +1205,11 @@ case 22:
 	}
     break;
 case 23:
-#line 205 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 205 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {getrusage(RUSAGE_SELF, &start_time);}
     break;
 case 24:
-#line 205 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 205 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {
 		getrusage(RUSAGE_SELF, &end_time);
 		cout << (end_time.ru_utime.tv_sec - start_time.ru_utime.tv_sec) +
@@ -1219,47 +1219,47 @@ case 24:
 	}
     break;
 case 25:
-#line 212 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 212 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {yyclearin; yyerrok;}
     break;
 case 26:
-#line 213 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 213 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {yyclearin; yyerrok;}
     break;
 case 27:
-#line 216 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 216 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {yyval = yyvsp[0];}
     break;
 case 28:
-#line 217 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 217 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {yyval = yyvsp[0].eval();}
     break;
 case 29:
-#line 218 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 218 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {yyval = yyvsp[-1];}
     break;
 case 30:
-#line 219 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 219 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {yyval = yyvsp[0];}
     break;
 case 31:
-#line 220 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 220 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {yyval = yyvsp[0];}
     break;
 case 32:
-#line 221 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 221 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {yyval = exstack[0];}
     break;
 case 33:
-#line 222 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 222 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {yyval = exstack[1];}
     break;
 case 34:
-#line 223 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 223 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {yyval = exstack[2];}
     break;
 case 35:
-#line 224 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 224 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {
 		fcn_tab::const_iterator i = find_function(yyvsp[-3], yyvsp[-1].nops());
 		if (i->second.is_ginac) {
@@ -1270,119 +1270,119 @@ case 35:
 	}
     break;
 case 36:
-#line 232 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 232 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {yyval = yyvsp[0]; Digits = ex_to<numeric>(yyvsp[0]).to_int();}
     break;
 case 37:
-#line 233 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 233 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {yyval = yyvsp[0]; const_cast<symbol&>(ex_to<symbol>(yyvsp[-2])).assign(yyvsp[0]);}
     break;
 case 38:
-#line 234 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 234 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {yyval = yyvsp[-2] == yyvsp[0];}
     break;
 case 39:
-#line 235 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 235 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {yyval = yyvsp[-2] != yyvsp[0];}
     break;
 case 40:
-#line 236 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 236 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {yyval = yyvsp[-2] < yyvsp[0];}
     break;
 case 41:
-#line 237 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 237 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {yyval = yyvsp[-2] <= yyvsp[0];}
     break;
 case 42:
-#line 238 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 238 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {yyval = yyvsp[-2] > yyvsp[0];}
     break;
 case 43:
-#line 239 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 239 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {yyval = yyvsp[-2] >= yyvsp[0];}
     break;
 case 44:
-#line 240 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 240 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {yyval = yyvsp[-2] + yyvsp[0];}
     break;
 case 45:
-#line 241 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 241 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {yyval = yyvsp[-2] - yyvsp[0];}
     break;
 case 46:
-#line 242 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 242 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {yyval = yyvsp[-2] * yyvsp[0];}
     break;
 case 47:
-#line 243 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 243 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {yyval = yyvsp[-2] / yyvsp[0];}
     break;
 case 48:
-#line 244 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 244 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {yyval = -yyvsp[0];}
     break;
 case 49:
-#line 245 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 245 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {yyval = yyvsp[0];}
     break;
 case 50:
-#line 246 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 246 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {yyval = power(yyvsp[-2], yyvsp[0]);}
     break;
 case 51:
-#line 247 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 247 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {yyval = factorial(yyvsp[-1]);}
     break;
 case 52:
-#line 248 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 248 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {yyval = yyvsp[-1];}
     break;
 case 53:
-#line 249 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 249 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {yyval = yyvsp[-1];}
     break;
 case 54:
-#line 250 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 250 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {yyval = lst_to_matrix(ex_to<lst>(yyvsp[-1]));}
     break;
 case 55:
-#line 253 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 253 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {yyval = exprseq(yyvsp[0]);}
     break;
 case 56:
-#line 254 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 254 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {exprseq es(ex_to<exprseq>(yyvsp[-2])); yyval = es.append(yyvsp[0]);}
     break;
 case 57:
-#line 257 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 257 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {yyval = *new lst;}
     break;
 case 58:
-#line 258 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 258 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {yyval = yyvsp[0];}
     break;
 case 59:
-#line 261 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 261 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {yyval = lst(yyvsp[0]);}
     break;
 case 60:
-#line 262 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 262 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {lst l(ex_to<lst>(yyvsp[-2])); yyval = l.append(yyvsp[0]);}
     break;
 case 61:
-#line 265 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 265 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {yyval = lst(yyvsp[-1]);}
     break;
 case 62:
-#line 266 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 266 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {lst l(ex_to<lst>(yyvsp[-4])); yyval = l.append(yyvsp[-1]);}
     break;
 case 63:
-#line 269 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 269 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {yyval = lst(yyvsp[0]);}
     break;
 case 64:
-#line 270 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 270 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 {lst l(ex_to<lst>(yyvsp[-2])); yyval = l.append(yyvsp[0]);}
     break;
 }
@@ -1618,7 +1618,7 @@ yyreturn:
 #endif
   return yyresult;
 }
-#line 278 "/home/cbauer/projects/GiNaC-1.0/ginsh/ginsh_parser.yy"
+#line 278 "/home/cbauer/projects/GiNaC-1.1/ginsh/ginsh_parser.yy"
 
 // Error print routine
 int yyerror(char *s)
@@ -1700,9 +1700,9 @@ static ex f_determinant(const exprseq &e)
 
 static ex f_diag(const exprseq &e)
 {
-	unsigned dim = e.nops();
+	size_t dim = e.nops();
 	matrix &m = *new matrix(dim, dim);
-	for (unsigned i=0; i<dim; i++)
+	for (size_t i=0; i<dim; i++)
 		m.set(i, i, e.op(i));
 	return m;
 }
@@ -1977,6 +1977,10 @@ static const fcn_help_init builtin_help[] = {
 	{"zeta", "zeta function\nzeta(x) is Riemann's zeta function, zeta(n,x) its nth derivative"},
 	{"Li2", "dilogarithm"},
 	{"Li3", "trilogarithm"},
+	{"Li", "(multiple) polylogarithm"},
+	{"S", "Nielsen's generalized polylogarithm"},
+	{"H", "harmonic polylogarithm"},
+	{"mZeta", "multiple zeta value"},
 	{"Order", "order term function (for truncated power series)"},
 	{"Derivative", "inert differential operator"},
 	{NULL, NULL}	// End marker
